// 2014-03-11 Sebastian Rockel (rockel@informatik.uni-hamburg.de)
//
#include <ros/ros.h>
#include "sim_manipulator.h"
#include <std_msgs/Empty.h>

class ObjectReset
{
  protected:
    std::string spawn_object;
    std::string pr2_gazebo_name;
    ros::NodeHandle nh;
    ros::Publisher reset_pub;

  public:
    ObjectReset(ros::Publisher & pub,
        std::string obj="race_flowers1", std::string rob="race_pr2")
    {
      reset_pub = pub;
      spawn_object = obj;
      pr2_gazebo_name = rob;
    }

    void resetOnTray(std::string obj="race_flowers1", std::string rob="race_pr2")
    {
      // Pose of object relative to the robots pose
      tf::Transform robot_tray = tf::Transform(tf::Quaternion(0,0,0,1), tf::Vector3(0.28, 0.0, 0.35));

      pr2_gazebo_name = rob;
      spawn_object = obj;

      // Set Gazebo to RACE map transform
      SimManipulator simManipulator(nh,
          tf::Transform(tf::Quaternion(0, 0, 0, 1), tf::Vector3(-12.28, -10.20, 0.0)));

      // Update the AMCL localization with the pose from the simulation
      //simManipulator.updateAmclFromSimulation(pr2_gazebo_name);
      // Get robot pose
      gazebo_msgs::ModelState robot_model;
      robot_model.model_name = pr2_gazebo_name;
      simManipulator.getModelPose(robot_model);

      //convert  gazebo_msgs::ModelState to  tf::Pose
      tf::Transform gaz_robot = tf::Transform(
          tf::Quaternion(
            robot_model.pose.orientation.x,
            robot_model.pose.orientation.y,
            robot_model.pose.orientation.z,
            robot_model.pose.orientation.w),
          tf::Vector3(
            robot_model.pose.position.x,
            robot_model.pose.position.y,
            robot_model.pose.position.z)
          );

      //transform object pose
      tf::Transform gaz_tray = gaz_robot * robot_tray;

      //
      if (simManipulator.modelExistsInSimulation(spawn_object))
      {
        //move object
        gazebo_msgs::ModelState move_object;
        move_object.model_name = spawn_object;
        move_object.pose.orientation.x = gaz_tray.getRotation().getX();
        move_object.pose.orientation.y = gaz_tray.getRotation().getY();
        move_object.pose.orientation.z = gaz_tray.getRotation().getZ();
        move_object.pose.orientation.w = gaz_tray.getRotation().getW();
        move_object.pose.position.x = gaz_tray.getOrigin().getX();
        move_object.pose.position.y = gaz_tray.getOrigin().getY();
        move_object.pose.position.z = gaz_tray.getOrigin().getZ();
        simManipulator.moveObject(move_object);
      }
      else
      {
        //spawn object
        FluentSimObject spawn_model;
        spawn_model.name = spawn_object;
        spawn_model.pose_map = gaz_tray;
        simManipulator.spawnFromFluent(spawn_model);
      }
    }

    void callback(const geometry_msgs::Vector3Stamped::ConstPtr& msg)
    {
      if(msg->vector.x > 0.5 || msg->vector.y > 0.1)
      {
        resetOnTray(spawn_object);
        reset_pub.publish(std_msgs::Empty());
      }
    }
};

int main( int argc, char** argv )
{
  ros::init(argc, argv, "gazebo_object_reset");
  ros::NodeHandle n;
  std::string object_name = "race_flowers1";

  ros::Publisher reset_pub = n.advertise<std_msgs::Empty>("obj_reset", 1);

  ros::Duration(10).sleep();
  ObjectReset object_reset(reset_pub, object_name);
  ros::Subscriber sub = n.subscribe("/object_monitor/obj_dev", 1000, &ObjectReset::callback, &object_reset);

  ros::spin();

  return 0;
}
