/*
 * 2014-04-25 Sebastian Rockel (rockel@informatik.uni-hamburg.de)
 *
 * Schedules the launch of the RACE Gazebo world
 */
#include <boost/thread.hpp>
#include <boost/algorithm/string.hpp>
#include "ros/ros.h"
#include "ros/package.h"


// TODO stop only the process requested
bool stopRos(void)
{
  bool success = true;

  // ROS is not alive anymore here
  success &= system("killall gzclient &>> /dev/null");
  success &= system("rosnode kill -a &>> /dev/null");
  sleep(5);
  success &= system("killall gzserver &>> /dev/null");
  sleep(5);
  success &= system("killall roscore &>> /dev/null");

  return success;
}

int main(int argc, char **argv)
{
  boost::thread_group th_group;
  std::string args;

  ros::init(argc, argv, "race_world_pr2");
  ros::NodeHandle nh;

  for (int i=1; i < argc; i++)
  {
    args.append(" " + std::string(argv[i]));
  }
  boost::replace_all(args, "=", ":=");
  ROS_INFO("Gazebo arguments: %s", args.c_str());

  th_group.create_thread(boost::bind(&system,
        "roslaunch race_gazebo_worlds race_world.launch gui:=false furniture:=false"));
  sleep(5);

  std::string items = "roslaunch race_gazebo_worlds spawn_scenario_items.launch " + args;
  ROS_INFO("%s",items.c_str());
  th_group.create_thread(boost::bind(&system, items.c_str()));
  if (args.find("gui:=true") != std::string::npos)
    th_group.create_thread(boost::bind(&system, "rosrun gazebo gui"));
  sleep(5);

  std::string spawnpr2 = "roslaunch race_gazebo_worlds spawn_pr2.launch " + args;
  ROS_INFO("%s", spawnpr2.c_str());
  th_group.create_thread(boost::bind(&system,
        spawnpr2.c_str()));
  sleep(5);

  //th_group.create_thread(boost::bind(&system, "rosrun race_simulation_run sim_amcl_update"));

  th_group.create_thread(boost::bind(&system,
        "rosrun dynamic_reconfigure dynparam set /gazebo max_update_rate 1000"));

  th_group.create_thread(boost::bind(&system,
        "roslaunch race_tachometer pr2_tachometer.launch"));

  th_group.create_thread(boost::bind(&system, "rosservice call /gazebo/unpause_physics '{}'"));

  /* Initialize ROS node */
  ros::spin();

  sleep(5);
  stopRos();
  th_group.interrupt_all();

  return 0;
}
